<?php

namespace Drupal\telephone_advanced\Plugin\Field\FieldType;

use Drupal\Core\Config\Entity\ThirdPartySettingsInterface;
use Drupal\telephone\Plugin\Field\FieldType\TelephoneItem as TelephoneTelephoneItem;
use Drupal\telephone_advanced\TelephoneFormatterInterface;
use libphonenumber\PhoneNumberFormat;

/**
 * Overrides the default telephone field class.
 */
class TelephoneItem extends TelephoneTelephoneItem {

  /**
   * The telephone formatter.
   *
   * @var \Drupal\telephone_advanced\TelephoneFormatterInterface|null
   */
  protected $telephoneFormatter;

  /**
   * {@inheritdoc}
   */
  public function preSave() {
    $storage_format = $this->getStorageFormat();

    if ($storage_format === NULL) {
      return;
    }

    $value = $this->getTelephoneFormatter()->format(
      $this->getValue()['value'],
      $storage_format,
      $this->getFieldDefinition()->getThirdPartySetting('telephone_advanced', 'default_country')
    );

    $this->set('value', $value, FALSE);
  }

  /**
   * Set the telephone formatter.
   *
   * @param \Drupal\telephone_advanced\TelephoneFormatterInterface $telephone_formatter
   *   The telephone formatter.
   */
  public function setTelephoneFormatter(TelephoneFormatterInterface $telephone_formatter) {
    $this->telephoneFormatter = $telephone_formatter;
  }

  /**
   * Get the telephone formatter.
   *
   * @return \Drupal\telephone_advanced\TelephoneFormatterInterface
   *   The telephone formatter.
   */
  protected function getTelephoneFormatter() {
    if (!isset($this->telephoneFormatter)) {
      /** @var \Drupal\telephone_advanced\TelephoneFormatterInterface $formatter */
      $formatter = \Drupal::service('telephone_advanced.telephone_formatter');
      $this->telephoneFormatter = $formatter;
    }

    return $this->telephoneFormatter;
  }

  /**
   * Get the telephone number storage format.
   *
   * @return int|null
   *   The storage format or NULL to skip formatting.
   */
  protected function getStorageFormat() {
    $field_definition = $this->getFieldDefinition();

    if (!$field_definition instanceof ThirdPartySettingsInterface) {
      return NULL;
    }

    if (!$field_definition->getThirdPartySetting('telephone_advanced', 'enabled', FALSE)) {
      return NULL;
    }

    return $field_definition->getThirdPartySetting('telephone_advanced', 'storage_format');
  }

}
